from . import BaseActor
import torch
import numpy as np

class BlockMSActor(BaseActor):
    def __init__(self, net, objective, loss_weight=None, stage='reg'):
        """
        args:
            net - The network to train
            objective - The loss function
        """
        super().__init__(net, objective, loss_weight)

        self.stage = stage

    """ Actor for training the TransT"""
    def __call__(self, data):
        """
        args:
            data - The input data, should contain the fields 'search_images', 'template_images', 'search_anno'.

        returns:
            loss    - the training loss
            states  -  dict containing detailed losses
        """
        import time
        outputs = self.net(data['search_images'], data['template_images'])#, self.stage)

        # generate labels
        targets =[]
        targets_origin = data['search_anno']
        for i in range(len(targets_origin)):
            h, w =data['search_images'][i][0].shape
            target_origin = targets_origin[i]
            target = {}
            target_origin = target_origin.reshape([1,-1])
            target_origin[0][0] += target_origin[0][2] / 2
            target_origin[0][0] /= w
            target_origin[0][1] += target_origin[0][3] / 2
            target_origin[0][1] /= h
            target_origin[0][2] /= w
            target_origin[0][3] /= h
            target['boxes'] = target_origin
            label = np.array([0])
            label = torch.tensor(label, device=data['search_anno'].device)
            target['labels'] = label
            targets.append(target)

        # Compute loss
        # outputs:(center_x, center_y, width, height)
        if isinstance(self.objective, dict):
            loss_dict = self.objective['tracking'](outputs, targets)
            weight_dict = self.objective['tracking'].weight_dict

        else:
            loss_dict = self.objective(outputs, targets)
            weight_dict = self.objective.weight_dict

        losses = sum(loss_dict[k] * weight_dict[k] for k in loss_dict.keys() if k in weight_dict)

        # Return training stats
        if self.stage =='reg':
            stats = {'reg_stage_Loss/total': losses.item(),
                     'Loss/bbox': loss_dict['loss_bbox'].item(),
                      'Loss/giou': loss_dict['loss_giou'].item(),
                      'iou': loss_dict['iou'].item()
                     }

        elif self.stage =='cls':
            stats = {'cls_stage_Loss/total': losses.item(),
                      'Loss/ce': loss_dict['loss_ce'].item(),
                     }

        elif self.stage == 'iou':
            stats = {'iou_stage_Loss/total': losses.item(),
                      'Loss/iou_predict_l1_loss': loss_dict['loss_l1_iou_predict'].item(),
                     }

        return losses, stats
