from ltr.actors import BaseActor
import torch
import numpy as np


def create_BCELogit_loss_label(label_size, pos_thr=16, neg_thr=24, upscale_factor=8):
    # Convert the thresholds if the stride is bigger than 1, else it stays the same.
    pos_thr = pos_thr / upscale_factor
    neg_thr = neg_thr / upscale_factor
    # Note that the center might be between pixels if the label size is even.
    center = (label_size - 1) / 2
    line = np.arange(0, label_size)
    line = line - center
    line = line**2
    line = np.expand_dims(line, axis=0)
    dist_map = line + line.transpose()

    label = np.zeros([label_size, label_size, 2]).astype(np.float32)
    label[:, :, 0] = dist_map <= pos_thr**2
    label[:, :, 1] = (dist_map <= pos_thr**2) | (dist_map > neg_thr**2)

    return label

def BCELogit_Loss(score_map, labels):
    loss = torch.nn.functional.binary_cross_entropy_with_logits(score_map, labels[:, 0, :, :], weight=labels[:, 1, :, :], reduction='mean')
    return loss

class SiamfcActor(BaseActor):
    """ Actor for training the TransT"""
    def __call__(self, data):
        """
        args:
            data - The input data, should contain the fields 'search_images', 'template_images', 'search_anno'.

        returns:
            loss    - the training loss
            states  -  dict containing detailed losses
        """
        import time
        outputs = self.net(data['search_images'], data['template_images'])
        output = outputs['pred_logits']
        b, c, h, w = output.shape

        labels = []
        for i in range(b):
           label = create_BCELogit_loss_label(label_size= 17, pos_thr=16, neg_thr=24, upscale_factor=8)
           label = torch.from_numpy(label)
           label = label.cuda()
           labels.append(label.unsqueeze(0))

        weight_dict = self.objective.weight_dict

        lables_c = torch.cat(labels, dim=0).permute(0, 3, 1, 2)
        losses = weight_dict['loss_ce'] * BCELogit_Loss(output.squeeze(1), lables_c)

        # Return training stats
        stats = {
            'Loss/total': losses.item(),
                 }

        return losses, stats



if __name__ == '__main__':
    #pth= '/home/v-xfei/Public/project/TransT/pytracking/networks/pvt_v2_b3.pth'
    label =  create_BCELogit_loss_label(label_size=17, pos_thr=16, neg_thr=24, upscale_factor=8)
    pos= label[:,:,0]



    a = 1