from . import BaseActor
import torch
import numpy as np

class Cor_BlockActor(BaseActor):
    """ Actor for training the TransT"""
    def __call__(self, data):
        """
        args:
            data - The input data, should contain the fields 'search_images', 'template_images', 'search_anno'.

        returns:
            loss    - the training loss
            states  -  dict containing detailed losses
        """
        outputs = self.net(data['search_images'], data['template_images'])  # xyxy normalized
        box_outputs = outputs['box']
        cls_outputs = outputs['cls']



        # Compute loss
        """
        boxes1: (N, 4) boxes2: (N, 4)
        """
        targets_origin = data['search_anno']
        targets_origin[:, 2] += targets_origin[:, 0]
        targets_origin[:, 3] += targets_origin[:, 1]  # xywh->xyxy

        corner_loss= self.objective['box']
        bce_loss= self.objective['cls']


        # compute loss
        reg_mse_loss = corner_loss(box_outputs, targets_origin)  # (BN,4) (BN,4)
        cls_bce_loss = bce_loss(cls_outputs, targets_origin)  # (BN,4) (BN,4)
        # weighted sum
        losses = self.loss_weight['box'] * reg_mse_loss + self.loss_weight['cls'] * cls_bce_loss

        # Return training stats
        stats = {'Loss/total': losses.item(),
                  # 'Loss/ce': loss_dict['loss_ce'].item(),
                  # 'Loss/l1': l1_loss.item(),
                 'Loss/mse_reg': reg_mse_loss.item(),
                 'Loss/bce_reg': cls_bce_loss.item(),

                 }

        return losses, stats
