from . import BaseActor
import torch
import numpy as np
import torch.nn.functional as F


def extract_Foreground( feat, anno):
    """
    anno = norm [x, y, w, h]
    """
    bs, c, h, w = feat.shape
    num_queries = h * w
    index_fg = []
    index_bg = []
    index_fg_bs = []
    index_bg_bs = []
    for i in range(bs):
        x, y, w, h = anno[i]
        x = x.item()
        y = y.item()
        w = w.item()
        h = h.item()
        xmin = x
        ymin = y
        xmax = x + w
        ymax = y + h
        len_feature = int(np.sqrt(num_queries))
        Xmin = int(np.ceil(xmin * len_feature))
        Ymin = int(np.ceil(ymin * len_feature))
        Xmax = int(np.ceil(xmax * len_feature))
        Ymax = int(np.ceil(ymax * len_feature))
        if Xmin == Xmax:
            Xmax = Xmax + 1
        if Ymin == Ymax:
            Ymax = Ymax + 1
        a = np.arange(0, num_queries, 1)
        b = a.reshape([len_feature, len_feature])
        index = torch.as_tensor(b, dtype=torch.int64)
        mask = torch.zeros(len_feature, len_feature).ge(0.5)
        mask[Ymin:Ymax, Xmin:Xmax] = True
        fg = torch.masked_select(index, mask)
        bg = torch.masked_select(index, ~mask)

        index_fg.append(fg)
        index_fg_bs.append(i * torch.ones(fg.shape, dtype=torch.int64))
        index_bg.append(bg)
        index_bg_bs.append(i * torch.ones(bg.shape, dtype=torch.int64))

    index_fg_bs = torch.cat(index_fg_bs, dim=0)
    # index_bg_bs = torch.cat(index_bg_bs, dim=0)
    index_fg = torch.cat(index_fg, dim=0)
    # index_bg = torch.cat(index_bg, dim=0)

    feat_rp = feat.reshape(bs, c, -1).permute(0, 2, 1)  # [bs, hw, c]
    feat_fg = feat_rp[(index_fg_bs, index_fg)]

    return feat_fg  # [b*n, c]

class QtActor(BaseActor):
    """ Actor for training the TransT"""
    def __call__(self, data):
        """
        args:
            data - The input data, should contain the fields 'search_images', 'template_images', 'search_anno'.

        returns:
            loss    - the training loss
            states  -  dict containing detailed losses
        """

        bs, c, h, w = data['template_images'].shape
        outputs = self.net(data['search_images'], data['template_images'], data['template_anno']/h)
        """
        #score = F.softmax(outputs['pred_logits'][0, ], dim=1).data[:, 0].reshape(18, -1).cpu().numpy()
        if 'reconst_abank' in outputs.keys():
            fg_abank = extract_Foreground(outputs['reconst_abank'], data['search_anno']/h) # [b*n, c]
            fg_dbank = extract_Foreground(outputs['reconst_dbank'], data['search_anno']/h) # [b*n, c]
            cosine_smilarity = F.cosine_similarity(fg_abank, fg_dbank, dim = 1)
            cos_sim_select = cosine_smilarity.ge(0)
            cos_sim_positive = cosine_smilarity.masked_select(cos_sim_select)
            cos_sim_loss = torch.mean(cos_sim_positive, dim= 0)
            loss_cosim = cos_sim_loss * 1
            if loss_cosim is N:
                loss_cosim= torch.tensor(0.0).cuda(), torch.tensor(0.0).cuda()
        """



        # generate labels
        targets = []
        targets_origin = data['search_anno']
        for i in range(len(targets_origin)):
            h, w = data['search_images'][i][0].shape
            target_origin = targets_origin[i]
            target = {}
            target_origin = target_origin.reshape([1, -1])
            target_origin[0][0] += target_origin[0][2] / 2
            target_origin[0][0] /= w
            target_origin[0][1] += target_origin[0][3] / 2
            target_origin[0][1] /= h
            target_origin[0][2] /= w
            target_origin[0][3] /= h
            target['boxes'] = target_origin
            label = np.array([0])
            label = torch.tensor(label, device=data['search_anno'].device)
            target['labels'] = label
            targets.append(target)

        # Compute loss
        # outputs:(center_x, center_y, width, height)

        loss_dict = self.objective(outputs, targets)
        weight_dict = self.objective.weight_dict

        losses = sum(loss_dict[k] * weight_dict[k] for k in loss_dict.keys() if k in weight_dict)
        #if 'reconst_abank' in outputs.keys():
          #  losses = losses + loss_cosim


        # Return training stats
        stats = {'Loss/total': losses.item(),
                 #'Loss/cosine_sim': loss_cosim.item(),
                 'Loss/ce': loss_dict['loss_ce'].item(),
                 'Loss/bbox': loss_dict['loss_bbox'].item(),
                 'Loss/giou': loss_dict['loss_giou'].item(),
                 'iou': loss_dict['iou'].item()
                 }

        return losses, stats
