from . import BaseActor
import torch
import numpy as np

class TokenTActor(BaseActor):
    """ Actor for training the TransT"""
    def __call__(self, data):
        """
        args:
            data - The input data, should contain the fields 'search_images', 'template_images', 'search_anno'.

        returns:
            loss    - the training loss
            states  -  dict containing detailed losses
        """
        import time
        outputs = self.net(data['search_images'], data['template_images'])
        pred_kp = outputs['pred_kp']  # [b, 4]  0-320

        # generate labels
        """
        targets =[]
        targets_origin = data['search_anno']
        for i in range(len(targets_origin)):
            h, w =data['search_images'][i][0].shape
            target_origin = targets_origin[i]
            target = {}
            target_origin = target_origin.reshape([1,-1])
            #target_origin[0][0] += target_origin[0][2] / 2
            target_origin[0][0] /= w
            #target_origin[0][1] += target_origin[0][3] / 2
            target_origin[0][1] /= h
            target_origin[0][2] /= w
            target_origin[0][3] /= h
            #target['boxes'] = target_origin
            #label = np.array([0])
            #label = torch.tensor(label, device=data['search_anno'].device)
            #target['labels'] = label
            targets.append(target_origin)

        """

        gt_boxes_vec = data['search_anno']  # [b, 4] xywh
        pred_boxes_vec = pred_kp

        try:
            """
             param boxes1: (N, 4) (x1,y1,x2,y2)
             param boxes2: (N, 4) (x1,y1,x2,y2)
            """
            giou_loss, iou = self.objective['giou'](pred_boxes_vec, gt_boxes_vec)  # (BN,4) (BN,4)
        except:
            giou_loss, iou = torch.tensor(0.0).cuda(), torch.tensor(0.0).cuda()
            # compute l1 loss
        l1_loss = self.objective['l1'](pred_boxes_vec, gt_boxes_vec)  # (BN,4) (BN,4)
        # weighted sum
        loss_weight = self.objective['weight']
        losses = loss_weight['giou'] * giou_loss + loss_weight['l1'] * l1_loss

        # Compute loss
        # outputs:(center_x, center_y, width, height)

        # Return training stats
        stats = {'Loss/total': losses.item(),
                 'Loss/giou':giou_loss.item(),
                 'Loss/l1': l1_loss.item(),
                 'iou': iou.item()
                 }

        return losses, stats


