from . import BaseActor
import torch
import numpy as np


def mask_iou(pred, target):

    """
    param: pred of size [N x H x W]
    param: target of size [N x H x W]
    """

    assert len(pred.shape) == 3 and pred.shape == target.shape

    N = pred.size(0)

    inter = torch.min(pred, target).sum(2).sum(1)
    union = torch.max(pred, target).sum(2).sum(1)

    iou = torch.sum(inter / union) / N

    return iou


def mask_iou_loss(pred, mask, num_object):

    N, K, H, W = mask.shape
    loss = torch.zeros(1).to(pred.device)
    start = 0 if K == num_object else 1

    iou_total = []
    for i in range(N):
        iou_i = mask_iou(pred[i, start:num_object+start], mask[i, start:num_object+start])
        loss += (1.0 - iou_i)
        iou_total = iou_i.clone().detach().cpu().numpy()


    loss = loss / N
    mean_iou = np.mean(iou_total)
    return loss, mean_iou


class TrackOnlyActor(BaseActor):
    """ Actor for training the TransT"""
    def __call__(self, data):
        """
        args:
            data - The input data, should contain the fields 'search_images', 'template_images', 'search_anno'.

        returns:
            loss    - the training loss
            states  -  dict containing detailed losses
            (data['search_images'], data['template_images'])
            search -> test
            template -> train
        """
        import time
        outputs = self.net(data['test_images'][0, :], data['train_images'][0,:], data['dym_images'][0, :])


        # generate labels
        targets =[]
        targets_origin = data['test_anno'][0,:].clone()
        for i in range(len(targets_origin)):
            h, w =data['test_images'][0][i][0].shape
            target_origin = targets_origin[i]
            target = {}
            target_origin = target_origin.reshape([1,-1])
            target_origin[0][0] += target_origin[0][2] / 2
            target_origin[0][0] /= w
            target_origin[0][1] += target_origin[0][3] / 2
            target_origin[0][1] /= h
            target_origin[0][2] /= w
            target_origin[0][3] /= h
            target['boxes'] = target_origin
            label = np.array([0])
            label = torch.tensor(label, device=data['test_anno'].device)
            target['labels'] = label
            targets.append(target)

        # Compute loss
        # outputs:(center_x, center_y, width, height)
        loss_dict = self.objective['tracking'](outputs, targets)
        weight_dict = self.objective['tracking'].weight_dict
        losses_tracking = sum(loss_dict[k] * weight_dict[k] for k in loss_dict.keys() if k in weight_dict)

        losses = losses_tracking

        # Return training stats
        stats = {'Loss/total': losses.item(),

                 'Loss/tracking': losses_tracking.item(),

                 'Loss/ce': loss_dict['loss_ce'].item(),
                 'Loss/bbox': loss_dict['loss_bbox'].item(),
                 'Loss/giou': loss_dict['loss_giou'].item(),

                 'iou': loss_dict['iou'].item(),
                 }

        return losses, stats
