import os
import sys
import argparse
import importlib
import multiprocessing
import cv2 as cv
import torch.backends.cudnn

from ltr.train_settings.SBTvot.SBTv1_jaf_rgbd import run

env_path = os.path.join(os.path.dirname(__file__), '..')
if env_path not in sys.path:
    sys.path.append(env_path)
import ltr.admin.settings as ws_settings

import torch.distributed as dist

def run_training(train_module, train_name, cudnn_benchmark=True,local_rank=None, type = 'base', debug_or = 0,  exp=0, resume=None, bz=None):
    """Run a train scripts in train_settings.
    args:
        train_module: Name of module in the "train_settings/" folder.
        train_name: Name of the train settings file.
        cudnn_benchmark: Use cudnn benchmark or not (default is True).
    """

    # This is needed to avoid strange crashes related to opencv
    cv.setNumThreads(0)

    torch.backends.cudnn.benchmark = cudnn_benchmark

    print('Training:  {}  {}'.format(train_module, train_name))

    settings = ws_settings.Settings()
    settings.module_name = train_module
    settings.script_name = train_name
    settings.project_path = 'ltr/{}/{}'.format(train_module, train_name)
    settings.local_rank=local_rank
    settings.device=local_rank
    settings.typemodel = type
    settings.debug = debug_or
    settings.exp = exp
    settings.resume = resume
    settings.bz = bz


   # expr_module = importlib.import_module('ltr.train_settings.{}.{}'.format(train_module, train_name))
    #expr_func = getattr(expr_module, 'run')
    #expr_module = importlib.import_module('ltr.train_settings.{}.{}'.format(train_module, train_name))
    run(settings)
    #expr_func(settings)


def main():
    parser = argparse.ArgumentParser(description='Run a train scripts in train_settings.')
    parser.add_argument('--train_module', type=str, default='SBTvot', help='Name of module in the "train_settings/" folder.')
    parser.add_argument('--train_name', type=str, default='SBTv1_jaf_rgbd', help='Name of the train settings file.')
    parser.add_argument('--cudnn_benchmark', type=bool, default=True, help='Set cudnn benchmark on (1) or off (0) (default is on).')
    parser.add_argument('--local_rank', type=int, default=-1, help='node rank for distrbuted trainging')
    parser.add_argument('--type', type=str, default='resnet50_depthwise', help='Name of the model.')
    parser.add_argument('--debug', type=int, default=0, help='Name of the model.')
    parser.add_argument('--exp', type=int, default=0, help='experiments.')
    parser.add_argument('--resume', type=str, default=None, help='resume pth path.')
    parser.add_argument('--bz', type=int, default=None, help='bz.')

    args = parser.parse_args()
    if args.local_rank != -1:
        dist.init_process_group(backend='nccl')
        torch.cuda.set_device(args.local_rank)
    else:
        torch.cuda.set_device(0)

    print('local rank: ', args.local_rank)

    run_training(args.train_module, args.train_name, args.cudnn_benchmark, args.local_rank, args.type, args.debug, args.exp, args.resume, args.bz)


if __name__ == '__main__':
    multiprocessing.set_start_method('spawn', force=True)
    main()
