from . import BaseActor
import torch
import numpy as np
from ltr.stark_train.box_ops import box_cxcywh_to_xyxy, box_xywh_to_xyxy

class Cor_BlockRegActor(BaseActor):
    """ Actor for training the TransT"""
    def __call__(self, data):
        """
        args:
            data - The input data, should contain the fields 'search_images', 'template_images', 'search_anno'.

        returns:
            loss    - the training loss
            states  -  dict containing detailed losses
        """
        outputs = self.net(data['search_images'][0,: ], data['template_images'][0,: ])  # xyxy normalized
        pred_boxes = outputs['box']
        #cls_outputs = outputs['cls']
        # Compute loss
        """
        boxes1: (N, 4) boxes2: (N, 4)
        """
        gt_bbox = data['search_anno'][0, ]
        gt_bbox[:, 2] += gt_bbox[:, 0]
        gt_bbox[:, 3] += gt_bbox[:, 1]  # xywh->xyxy

        pred_boxes_vec = box_cxcywh_to_xyxy(pred_boxes).view(-1, 4)  # (B,N,4) --> (BN,4) (x1,y1,x2,y2)
        gt_boxes_vec = box_xywh_to_xyxy(gt_bbox)#[:, None, :].repeat((1, num_queries, 1)).view(-1, 4).clamp(min=0.0,
                                                   #                                                        max=1.0)  # (B,4) --> (B,1,4) --> (B,N,4)
        # compute loss
        try:
            giou_loss, iou = self.objective['giou'](pred_boxes_vec, gt_boxes_vec)  # (BN,4) (BN,4)
        except:
            giou_loss, iou = torch.tensor(0.0).cuda(), torch.tensor(0.0).cuda()

            # compute l1 loss
        l1_loss = self.objective['l1'](pred_boxes_vec, gt_boxes_vec)  # (BN,4) (BN,4)
        # weighted sum
        loss = self.loss_weight['giou'] * giou_loss + self.loss_weight['l1'] * l1_loss

        mean_iou = iou.detach().mean()
        status = {      "Loss/total": loss.item(),
                        "Loss/giou": giou_loss.item(),
                        "Loss/l1": l1_loss.item(),
                        "IoU": mean_iou.item()
                         }
        return loss, status

"""
pred_boxes_vec = box_cxcywh_to_xyxy(pred_boxes).view(-1, 4)  # (B,N,4) --> (BN,4) (x1,y1,x2,y2)
        gt_boxes_vec = box_xywh_to_xyxy(gt_bbox)[:, None, :].repeat((1, num_queries, 1)).view(-1, 4).clamp(min=0.0, max=1.0)  # (B,4) --> (B,1,4) --> (B,N,4)
        # compute giou and iou
        try:
            giou_loss, iou = self.objective['giou'](pred_boxes_vec, gt_boxes_vec)  # (BN,4) (BN,4)
        except:
            giou_loss, iou = torch.tensor(0.0).cuda(), torch.tensor(0.0).cuda()
        # compute l1 loss
        l1_loss = self.objective['l1'](pred_boxes_vec, gt_boxes_vec)  # (BN,4) (BN,4)
        # weighted sum
        loss = self.loss_weight['giou'] * giou_loss + self.loss_weight['l1'] * l1_loss
        if return_status:
            # status for log
            mean_iou = iou.detach().mean()
            status = {"Loss/total": loss.item(),
                      "Loss/giou": giou_loss.item(),
                      "Loss/l1": l1_loss.item(),
                      "IoU": mean_iou.item()}
            return loss, status
        else:
            return loss

"""
