
from __future__ import absolute_import

import numpy as np
import math
import torchvision.transforms.functional as tvisf
import cv2
import torch
import torch.nn.functional as F
import time
from pysot_toolkit.trackers.vot20_utils import *
import pysot_toolkit.vot2020 as vot

import os
from pysot_toolkit.trackers.net_wrappers import NetWithBackbone
import numpy as np
import random
import matplotlib.pyplot as plt

class Tracker_dualTFRpp(object):

    def __init__(self, name, net, type='base', window_penalty=0.49, window_size=16, exemplar_size=128, instance_size=256,):
        self.name = name
        self.net = net
        self.exemplar_size = exemplar_size
        self.instance_size = instance_size
        self.window_size = int(window_size)
        self.type= type

        self.THRES = 0.65
        self.window_penalty = window_penalty

        self.penalty_k = 0.04
        self.test_lr = 0.95
        self.window_influence = 0.21

    def _convert_score(self, score):

        score = score.permute(2, 1, 0).contiguous().view(2, -1).permute(1, 0)
        score = F.softmax(score, dim=1).data[:, 0].cpu().numpy()
        return score

    def _convert_bbox(self, delta):

        delta = delta.permute(2, 1, 0).contiguous().view(4, -1)
        delta = delta.data.cpu().numpy()

        return delta

    def _bbox_clip(self, cx, cy, width, height, boundary):
        cx = max(0, min(cx, boundary[1]))
        cy = max(0, min(cy, boundary[0]))
        width = max(10, min(width, boundary[1]))
        height = max(10, min(height, boundary[0]))
        return cx, cy, width, height

    def get_subwindow(self, im, pos, model_sz, original_sz, avg_chans):
        """
        args:
            im: rgb based image
            pos: center position
            model_sz: exemplar size
            original_sz: original size
            avg_chans: channel average
        """
        if isinstance(pos, float):
            pos = [pos, pos]
        sz = original_sz
        im_sz = im.shape
        c = (original_sz + 1) / 2
        # context_xmin = round(pos[0] - c) # py2 and py3 round
        context_xmin = np.floor(pos[0] - c + 0.5)
        context_xmax = context_xmin + sz - 1
        # context_ymin = round(pos[1] - c)
        context_ymin = np.floor(pos[1] - c + 0.5)
        context_ymax = context_ymin + sz - 1
        left_pad = int(max(0., -context_xmin))
        top_pad = int(max(0., -context_ymin))
        right_pad = int(max(0., context_xmax - im_sz[1] + 1))
        bottom_pad = int(max(0., context_ymax - im_sz[0] + 1))

        context_xmin = context_xmin + left_pad
        context_xmax = context_xmax + left_pad
        context_ymin = context_ymin + top_pad
        context_ymax = context_ymax + top_pad

        r, c, k = im.shape
        if any([top_pad, bottom_pad, left_pad, right_pad]):
            size = (r + top_pad + bottom_pad, c + left_pad + right_pad, k)
            te_im = np.zeros(size, np.uint8)
            te_im[top_pad:top_pad + r, left_pad:left_pad + c, :] = im
            if top_pad:
                te_im[0:top_pad, left_pad:left_pad + c, :] = avg_chans
            if bottom_pad:
                te_im[r + top_pad:, left_pad:left_pad + c, :] = avg_chans
            if left_pad:
                te_im[:, 0:left_pad, :] = avg_chans
            if right_pad:
                te_im[:, c + left_pad:, :] = avg_chans
            im_patch = te_im[int(context_ymin):int(context_ymax + 1),
                       int(context_xmin):int(context_xmax + 1), :]
        else:
            im_patch = im[int(context_ymin):int(context_ymax + 1),
                       int(context_xmin):int(context_xmax + 1), :]

        if not np.array_equal(model_sz, original_sz):
            im_patch = cv2.resize(im_patch, (model_sz, model_sz))
        crop = im_patch.copy()
        im_patch = im_patch.transpose(2, 0, 1)
        im_patch = im_patch[np.newaxis, :, :, :]
        im_patch = im_patch.astype(np.float32)
        im_patch = torch.from_numpy(im_patch)
        im_patch = im_patch.cuda()
        return im_patch, crop


    def initialize_features(self,):
        #if not getattr(self, 'features_initialized', False):
            #self.net.initialize()
        self.features_initialized = True
        self.net.cuda()

    def initialize(self, image, mask):
        self.frame_num =1
        tic = time.time()
        hanning = np.hanning(self.window_size)  # 14
        window = np.outer(hanning, hanning)
        self.window = window.flatten()
        self.window_penalty_func = self.window
        # Initialize
        self.initialize_features()
        """"""""""""
        if isinstance(mask, list):
            region = mask
        elif isinstance(mask, dict):
            region = mask['init_bbox']
        else:
            region = rect_from_mask(mask)

        self.H, self.W, _ = image.shape
        gt_bbox_np = np.array(region).astype(np.float32)

        '''initilize refinement module for specific video'''
        #self.alpha.initialize(image, gt_bbox_np)

        bbox = gt_bbox_np
        self.center_pos = np.array([bbox[0] + bbox[2] / 2,
                                    bbox[1] + bbox[3] / 2])
        self.size = np.array([bbox[2], bbox[3]])

        # calculate z crop size
        w_z = self.size[0] + (2 - 1) * ((self.size[0] + self.size[1]) * 0.5)  #
        h_z = self.size[1] + (2 - 1) * ((self.size[0] + self.size[1]) * 0.5)
        s_z = math.ceil(math.sqrt(w_z * h_z))

        # calculate channle average
        self.channel_average = np.mean(image, axis=(0, 1))

        # get crop
        z_crop, imgz_crop = self.get_subwindow(image, self.center_pos,
                                    self.exemplar_size,
                                    s_z, self.channel_average)

        # normalize
        z_crop = z_crop.float().mul(1.0 / 255.0).clamp(0.0, 1.0)
        self.mean = [0.485, 0.456, 0.406]
        self.std = [0.229, 0.224, 0.225]
        self.inplace = False
        z_crop[0] = tvisf.normalize(z_crop[0], self.mean, self.std, self.inplace)

        # initialize template feature
        self.net.template(z_crop)
        out = {'time': time.time() - tic}
        self.z_c = z_crop
        return out

    def track(self, image, gt= None):

        self.size = [gt[2], gt[3]]
        cx_gt = gt[0] + gt[2] / 2
        cy_gt = gt[1] + gt[3] / 2


        # calculate x crop size
        w_x = self.size[0] + (4 - 1) * ((self.size[0] + self.size[1]) * 0.5)
        h_x = self.size[1] + (4 - 1) * ((self.size[0] + self.size[1]) * 0.5)
        s_x = math.ceil(math.sqrt(w_x * h_x))

        x1_norm = (s_x * 0.5 - self.size[0] * 0.5)/s_x
        y1_norm = (s_x * 0.5 - self.size[1] * 0.5)/s_x

        x2_norm = (s_x * 0.5 + self.size[0] * 0.5) / s_x
        y2_norm = (s_x * 0.5 + self.size[1] * 0.5) / s_x


        rand_cx = 0 #round(random.uniform(-0.7, 0.7), 2)
        rand_cy = 0  #round(random.uniform(-0.7, 0.7), 2)

        cx_gt_rand = cx_gt + (0.5*s_x) * rand_cx
        cy_gt_rand = cy_gt + (0.5*s_x) * rand_cy

        self.center_pos = [cx_gt_rand, cy_gt_rand]

        # get crop
        x_crop, img_x = self.get_subwindow(image, self.center_pos,
                                    self.instance_size,
                                    round(s_x), self.channel_average)

        img_x = cv2.cvtColor(img_x, cv2.COLOR_RGB2BGR)
        path = '/home/v-xfei/Public/project/TransT_result/vis_abla/'
        cv2.imwrite(path + str(self.frame_num)  + '_x' + str(rand_cx) + '_y' + str(rand_cy) +'.jpg' , img_x)

        # normalize
        x_crop = x_crop.float().mul(1.0 / 255.0).clamp(0.0, 1.0)
        x_crop[0] = tvisf.normalize(x_crop[0], self.mean, self.std, self.inplace)

        # track
        outputs = self.net.track(x_crop)
        score = self._convert_score(outputs['pred_logits'])
        score = score.reshape(32, -1)
        #score = score * (score > 0.5)
        mask_score = score > 0.6
        #import seaborn as sns
        #sns.heatmap(score, vmin=0, vmax =1, cmap='Blues')
        #plt.show()
        #sns.heatmap(mask_score, vmin=0, vmax=1, cmap='Blues')
        #plt.show()

        fused_out = outputs['fused_out'].cpu().squeeze(0)
        c, h, w = fused_out.shape

        out = {
            'score': score,
            'mask_score': mask_score,
            'img_x':img_x,
        }
        mask_score = torch.from_numpy(mask_score.reshape(-1))
        index = torch.from_numpy(np.arange(0, h*w, 1))
        index_pos = torch.masked_select(index, mask_score)
        index_neg = torch.masked_select(index, ~mask_score)

        fused_out = fused_out.reshape(c, -1).permute(1, 0)  # [hw, c]
        pos_f = fused_out[index_pos, :]
        pos_n = fused_out[index_neg, :]

        score_t = torch.from_numpy(score).reshape(-1)
        label = torch.where(score_t > 0.6, torch.ones_like(score_t), torch.zeros_like(score_t)).numpy()
        from sklearn.manifold import TSNE
        #data = fused_out #np.concatenate()    [n, c]
        #tsne = TSNE(n_components=2, init ='pca', random_state=0)
        #result = tsne.fit_transform(data)
        #fig = self.plot_embedding(result, label, 'tsne')
        #fig_path = '/home/v-xfei/Public/project/TransT_result/vis_abla/tsne/' + str(self.frame_num) + '.png'
        #plt.savefig(fig_path)

        gt_region = np.zeros_like(score)
        gt_region[int(y1_norm*32):int(y2_norm*32), int(x1_norm*32):int(x2_norm*32)] = 1
        # label distractor object
        x1_norm_2 = x1_norm + 0.2
        x2_norm_2 = x2_norm + 0.3
        y1_norm_2 = y1_norm - 0.1
        y2_norm_2 = y2_norm - 0.1
        #gt_region[int(y1_norm_2 *32):int(y2_norm_2 *32), int(x1_norm_2 *32):int(x2_norm_2 *32)] = 2

        #show = gt_region.copy().reshape(32, -1)
        #val_img = self.vis_heatmap( img_x, show*0.2)
        #path = '/home/v-xfei/Public/project/TransT_result/vis_abla/'
        #cv2.imwrite(path + 'val.jpg', val_img)

        label = gt_region.reshape(-1)

        for i in range(9):
            fea = outputs['interout'][i].cpu().squeeze(0)
            fea = fea.reshape(c, -1).permute(1, 0)  # [hw, c]
            tsne = TSNE(n_components=2, init='pca', random_state=0)
            result = tsne.fit_transform(fea)
            fig = self.plot_embedding(result, label, 'tsne')
            fig_path = '/home/v-xfei/Public/project/TransT_result/vis_abla/fig_tsne_' + str(self.frame_num)+'_'+str(i) + '.png'
            plt.savefig(fig_path)

        #plt.show()

        vis_fig = self.vis_heatmap(img_x, score)
        path = '/home/v-xfei/Public/project/TransT_result/vis_abla/'

        cv2.imwrite(path + str(self.frame_num) + '_x' + str(rand_cx) + '_y' + str(rand_cy) + '_heatmap.jpg', vis_fig)

        self.frame_num +=1
        return out

    def plot_embedding(self, data, label, title):
        x_min, x_max = np.min(data, 0), np.max(data, 0)
        data = (data- x_min)/(x_max- x_min)

        color_dict={
            0: "#FA8072",
            1: "#00FF00", # green
            2: "#0000FF",  # blue

        }
        str_dict={
            0: 'B',
            1: 'F',
            2: 'D',

        }

        fig = plt.figure()
        ax = plt.subplot(111)
        for i in range(data.shape[0]):
            plt.text(data[i, 0], data[i, 1], str_dict[label[i]], color=color_dict[label[i]],  # plt.cm.Set1((color_dict[label[i]]))
                     fontdict={'weight': 'bold', 'size':9})
        plt.xticks([])
        plt.yticks([])
       # plt.title(title)
        return fig

    def vis_heatmap(self, img, att):
        import cv2
        import numpy as np

        """ 
        img: np.ndarray (uint8) in shape of [H, W, 3]. 
        att: np.ndarray (float32) in shape of [H, W], ranged [0, 1]
        """
        h, w = img.shape[0:2]
        att = cv2.resize(att, (w, h), interpolation=cv2.INTER_CUBIC)
        att = np.clip(att, 0, 1)

        att_uint8 = (att * 255).astype(np.uint8)
        att_vis = cv2.applyColorMap(att_uint8, cv2.COLORMAP_JET)

        img = img.astype(np.float32)
        att_vis = att_vis.astype(np.float32)

        out = (1 - att ** 0.8).reshape(h, w, 1) * img + \
              (att ** 0.8).reshape(h, w, 1) * att_vis

        out = np.clip(out, 0, 255).astype(np.uint8)
        return out

    def _bbox_clip_xyxy(self, x1, y1, x2, y2, boundary, min_sz=10):
        '''boundary (H,W)'''
        x1_new = max(0, min(x1, boundary[1] - min_sz))
        y1_new = max(0, min(y1, boundary[0] - min_sz))
        x2_new = max(min_sz, min(x2, boundary[1]))
        y2_new = max(min_sz, min(y2, boundary[0]))
        return x1_new, y1_new, x2_new, y2_new

    def _postprocess_score(self, score, box_wh, target_sz, scale_x):
        r"""
        Perform SiameseRPN-based tracker's post-processing of score
        :param score: (HW, ), score prediction
        :param box_wh: (HW, 4), cxywh, bbox prediction (format changed)
        :param target_sz: previous state (w & h)
        :param scale_x:
        :return:
            best_pscore_id: index of chosen candidate along axis HW
            pscore: (HW, ), penalized score
            penalty: (HW, ), penalty due to scale/ratio change
        """
        def change(r):
            return np.maximum(r, 1. / r)

        def sz(w, h):
            pad = (w + h) * 0.5
            sz2 = (w + pad) * (h + pad)
            return np.sqrt(sz2)

        def sz_wh(wh):
            pad = (wh[0] + wh[1]) * 0.5
            sz2 = (wh[0] + pad) * (wh[1] + pad)
            return np.sqrt(sz2)

        # size penalty
        penalty_k = self.penalty_k
        target_sz_in_crop = [target_sz[0] , target_sz[1] ]
        s_c = change(
            sz(box_wh[2, :]*scale_x, box_wh[3, :]*scale_x) /
            (sz_wh(target_sz_in_crop)))  # scale penalty
        r_c = change((target_sz_in_crop[0] / target_sz_in_crop[1]) /
                     (box_wh[2, :] / box_wh[3, :]))  # ratio penalty
        penalty = np.exp(-(r_c * s_c - 1) * penalty_k)
        pscore = penalty * score

        # ipdb.set_trace()
        # cos window (motion model)
        window_influence = self.window_influence
        pscore = pscore * (
            1 - window_influence) + self.window_penalty_func * window_influence
        best_pscore_id = np.argmax(pscore)

        return best_pscore_id, pscore, penalty

    def _postprocess_box(self, best_pscore_id, score, box_wh, target_pos,
                         target_sz, scale_x, penalty):
        r"""
        Perform SiameseRPN-based tracker's post-processing of box
        :param score: (HW, ), score prediction
        :param box_wh: (HW, 4), cxywh, bbox prediction (format changed)
        :param target_pos: (2, ) previous position (x & y)
        :param target_sz: (2, ) previous state (w & h)
        :param scale_x: scale of cropped patch of current frame
        :param x_size: size of cropped patch
        :param penalty: scale/ratio change penalty calculated during score post-processing
        :return:
            new_target_pos: (2, ), new target position
            new_target_sz: (2, ), new target size
        """
        pred_in_crop = box_wh[:, best_pscore_id] * np.float32(scale_x)
        # about np.float32(scale_x)
        # attention!, this casting is done implicitly
        # which can influence final EAO heavily given a model & a set of hyper-parameters

        # box post-postprocessing
        test_lr = self.test_lr
        lr = penalty[best_pscore_id] * score[best_pscore_id] * test_lr
        res_x = pred_in_crop[0] + target_pos[0] - scale_x / 2 #pred_in_crop[0] + target_pos[0] - (x_size // 2) / scale_x
        res_y = pred_in_crop[1] + target_pos[1] - scale_x / 2 #pred_in_crop[1] + target_pos[1] - (x_size // 2) / scale_x
        res_w = target_sz[0] * (1 - lr) + pred_in_crop[2] * lr
        res_h = target_sz[1] * (1 - lr) + pred_in_crop[3] * lr

        return res_x, res_y, res_w, res_h

    def _restrict_box(self, target_pos, target_sz):
        r"""
        Restrict target position & size
        :param target_pos: (2, ), target position
        :param target_sz: (2, ), target size
        :return:
            target_pos, target_sz
        """
        target_pos[0] = max(0, min(self._state['im_w'], target_pos[0]))
        target_pos[1] = max(0, min(self._state['im_h'], target_pos[1]))
        target_sz[0] = max(self._hyper_params['min_w'],
                           min(self._state['im_w'], target_sz[0]))
        target_sz[1] = max(self._hyper_params['min_h'],
                           min(self._state['im_h'], target_sz[1]))

        return target_pos, target_sz

    def _cvt_box_crop2frame(self, box_in_crop, target_pos, scale_x, x_size):
        r"""
        Convert box from cropped patch to original frame
        :param box_in_crop: (4, ), cxywh, box in cropped patch
        :param target_pos: target position
        :param scale_x: scale of cropped patch
        :param x_size: size of cropped patch
        :return:
            box_in_frame: (4, ), cxywh, box in original frame
        """
        x = (box_in_crop[..., 0]) / scale_x + target_pos[0] - (x_size //
                                                               2) / scale_x
        y = (box_in_crop[..., 1]) / scale_x + target_pos[1] - (x_size //
                                                               2) / scale_x
        w = box_in_crop[..., 2] / scale_x
        h = box_in_crop[..., 3] / scale_x
        box_in_frame = np.stack([x, y, w, h], axis=-1)

        return box_in_frame















def run_vot_exp(VIS=False):

    torch.set_num_threads(1)
    # torch.cuda.set_device(CUDA_ID)  # set GPU id
    save_root = '/home/v-xfei/Public/dataset/vot_tools/vot21/vis/'
    if VIS and (not os.path.exists(save_root)):
        os.mkdir(save_root)
    net_path = '/home/v-xfei/Public/project/TransT/pytracking/networks/CrossViT_ep0046.pth.tar'  # Absolute path of the model

    # create model
    net = NetWithBackbone(net_path=net_path, use_gpu=True)
    tracker = TrackingNC_ALPHA_box(name='dualtfr160', net=net, window_penalty=0.49, exemplar_size=224, instance_size=224)

    handle = vot.VOT("mask")
    selection = handle.region()
    imagefile = handle.frame()

    if VIS:
        '''for vis'''
        seq_name = imagefile.split('/')[-3]
        save_v_dir = os.path.join(save_root,seq_name)
        if not os.path.exists(save_v_dir):
            os.mkdir(save_v_dir)
        cur_time = int(time.time() % 10000)
        save_dir = os.path.join(save_v_dir, str(cur_time))
        if not os.path.exists(save_dir):
            os.mkdir(save_dir)

    image = cv2.cvtColor(cv2.imread(imagefile), cv2.COLOR_BGR2RGB) # Right
    # mask given by the toolkit ends with the target (zero-padding to the right and down is needed)
    mask = make_full_size(selection, (image.shape[1], image.shape[0]))
    tracker.initialize(image, mask)

    while True:
        imagefile = handle.frame()
        if not imagefile:
            break
        image = cv2.cvtColor(cv2.imread(imagefile), cv2.COLOR_BGR2RGB)  # Right
        b1, m = tracker.track(image)
        handle.report(m)
        if VIS:
            '''Visualization'''
            image_show = image[:,:,::-1].copy() # RGB --> BGR
            image_show = image_show.astype(np.float32)

            image_show[:,:,0] += 127.0 * m
            image_show = image_show.clip(0,255).astype(np.uint8)
            cv2.rectangle(image_show,(int(b1[0]),int(b1[1])),
                          (int(b1[0]+b1[2]),int(b1[1]+b1[3])),(0,0,255),2)
            save_path = os.path.join(save_dir, imagefile.split('/')[-1])
            cv2.imwrite(save_path,image_show)

#run_vot_exp()