# Copyright (c) SenseTime. All Rights Reserved.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import os
import sys
env_path = os.path.join(os.path.dirname(__file__), '..')
print(env_path)
if env_path not in sys.path:
    sys.path.append(env_path)
import argparse
import os

import cv2
import torch
import numpy as np

from pysot_toolkit.bbox import get_axis_aligned_bbox
from pysot_toolkit.toolkit.datasets import DatasetFactory
from pysot_toolkit.toolkit.utils.region import vot_overlap, vot_float2str
from pysot_toolkit.trackers.tracker import Tracker
from pysot_toolkit.trackers.tracker_blockT_detr import Tracker_blockT
from pysot_toolkit.trackers.vot20_utils import *
import random

import matplotlib.pyplot as plt
torch.set_grad_enabled(False);

def set_seed(seed):
    random.seed(seed)
    os.environ['PYTHONHASHSEED'] = str(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.cuda.manual_seed(seed)
    torch.backends.cudnn.benchmark = True
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.enabled = True

def vis_heatmap(img, att):
    import cv2
    import numpy as np

    """ 
    img: np.ndarray (uint8) in shape of [H, W, 3]. 
    att: np.ndarray (float32) in shape of [H, W], ranged [0, 1]
    """
    h, w = img.shape[0:2]
    att = cv2.resize(att, (w, h), interpolation=cv2.INTER_CUBIC)
    att = np.clip(att, 0, 1)

    att_uint8 = (att * 255).astype(np.uint8)
    att_vis = cv2.applyColorMap(att_uint8, cv2.COLORMAP_JET)

    img = img.astype(np.float32)
    att_vis = att_vis.astype(np.float32)

    out = (1 - att ** 0.8).reshape(h, w, 1) * img + \
          (att ** 0.8).reshape(h, w, 1) * att_vis

    out = np.clip(out, 0, 255).astype(np.uint8)
    return out

def main():
    seed_test = 1000
    set_seed(seed_test)
    # load config
    video_show =['basketball']# ['crabs1', 'basketball','bag', 'ants1','car1' 'butterfly'] #'basketball' # bag
    vis = False
    dataset = 'VOT2018'
    if '18' in dataset:
        dataset = 'VOT2018'
        dataset_root = '/home/v-xfei/Public/dataset/vot_tools/vot18/sequences/'
    elif '19' in dataset:
        dataset = 'VOT2019'
        dataset_root = '/home/v-xfei/Public/dataset/vot_tools/vot19/sequences/'
    elif 'GOT' in dataset:
        dataset = 'GOT-10k'
        dataset_root = '/home/v-xfei/Public/dataset/vottrain/got10k/test/'
    elif 'val' in dataset:
        dataset = 'GOT-10k'
        dataset_root = '/home/v-xfei/Public/dataset/vottrain/got10k/val/'
    elif 'jq' in dataset:
        dataset = 'GOT-10k'
        dataset_root = '/mnt/data/vottrain/got10k/test/'

    name = 'pvt_begin_lr_75'
    net_path = '/home/v-xfei/Public/project/TransT_result/networks/begin/pvt_v2_b3_ep0075.pth.tar'
    output = './'
    from ltr.models.blockT1.pvtv2b3_1 import build_pvtv2b3_tracker
    crossdict = [
        [2],
        [2, 4],
        [2, 4, 6, 8, 10],
    ]
    model = build_pvtv2b3_tracker(crossdict)
    model.load_state_dict(torch.load(net_path)['net'], strict=False)
    model.cuda()
    # create model
    DualTFRpp = Tracker_blockT(name='Tracker_blockT', net_path=net_path,  type=type, window_penalty=0.49,
                               window_size =32, exemplar_size=128, instance_size=256, net=model)
    tracker = DualTFRpp

    # create dataset
    dataset = DatasetFactory.create_dataset(name=dataset,
                                            dataset_root=dataset_root,
                                            load_img=False)

    model_name = name
    total_lost = 0
    for v_idx, video in enumerate(dataset):
            if video_show != '':
                # test one special video
                if video.name not in video_show:
                    continue
            toc = 0
            pred_bboxes = []
            scores = []
            track_times = []
            for idx, (img, gt_bbox) in enumerate(video):
                img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                tic = cv2.getTickCount()
                if idx == 0:
                    cx, cy, w, h = get_axis_aligned_bbox(np.array(gt_bbox))
                    gt_bbox_ = [cx-w/2, cy-h/2, w, h]
                    init_info = {'init_bbox':gt_bbox_}
                    tracker.initialize(img, init_info)
                    pred_bbox = gt_bbox_
                    scores.append(None)
                    if 'VOT2018-LT' == dataset:
                        pred_bboxes.append([1])
                    else:
                        pred_bboxes.append(pred_bbox)
                else:
                    model = tracker.net
                    vis_1, vis_2, vis_3, vis_4, vis_5, vis_6, vis_7, vis_8, vis_9, vis_10 = [], [], [], [], [],[], [],[], [],[]
                    hooks = [
                        model.block3[0].attn.register_forward_hook(
                            lambda self, input, output: vis_1.append(output)
                        ),
                        model.block3[1].attn.register_forward_hook(
                            lambda self, input, output: vis_2.append(output)
                        ),
                        model.block3[2].attn.register_forward_hook(
                            lambda self, input, output: vis_3.append(output)
                        ),
                        model.block3[3].attn.register_forward_hook(
                            lambda self, input, output: vis_4.append(output)
                        ),
                        model.block3[4].attn.register_forward_hook(
                            lambda self, input, output: vis_5.append(output)
                        ),
                        model.block3[5].attn.register_forward_hook(
                            lambda self, input, output: vis_6.append(output)
                        ),
                        model.block3[6].attn.register_forward_hook(
                            lambda self, input, output: vis_7.append(output)
                        ),
                        model.block3[7].attn.register_forward_hook(
                            lambda self, input, output: vis_8.append(output)
                        ),
                        model.block3[8].attn.register_forward_hook(
                            lambda self, input, output: vis_9.append(output)
                        ),
                        model.block3[9].attn.register_forward_hook(
                            lambda self, input, output: vis_10.append(output)
                        )
                    ]

                    outputs = tracker.track(img) #, gt_bbox_)

                    for hook in hooks:
                        hook.remove()

                    vis_1 = vis_1[0]
                    vis_2 = vis_2[0]
                    vis_3 = vis_3[0]
                    vis_4 = vis_4[0]
                    vis_5 = vis_5[0]
                    vis_6 = vis_6[0]
                    vis_7 = vis_7[0]
                    vis_8 = vis_8[0]
                    vis_9 = vis_9[0]
                    vis_10 = vis_10[0]

                    vis = [vis_1, vis_2, vis_3, vis_4, vis_5, vis_6, vis_7, vis_8, vis_9, vis_10]

                    flag = ''
                    for i in range(10):
                        if len(vis[i]) ==2 :
                            show = vis[i][-1]['self_x']  # [1, head, :, 7]
                            flag = 'self'
                            # i = 165
                            show1 = show[0,4, 512]
                            hw = show1.size(0)
                            h = int(hw ** 0.5)
                            attn1 = show1.view(h, -1).detach().cpu().numpy()

                            image_x = tracker.image_x  # image_x
                            b1 = outputs['bbox_crop']

                            fused_img = vis_heatmap(image_x[:, :, ::-1].copy(), attn1 * 40)

                            cv2.rectangle(fused_img, (int(b1[0] - 0.5 * b1[2]), int(b1[1] - 0.5 * b1[3])),
                                          (int(b1[0] + 0.5 * b1[2]), int(b1[1] + 0.5 * b1[3])), (0, 0, 255), 1)

                            video_path = '/home/v-xfei/Public/vis/a_dualtfrpp/' + video.name
                            if not os.path.isdir(video_path):
                                os.makedirs(video_path)
                            cv2.imwrite(video_path + '/' + str(idx) + '_selfx_' + str(i+1) + '_fused.jpg', fused_img)

                        else:
                            show_xz = vis[i][-1]['cross_xz']  # [0, 0, :, 7]
                            show_zx = vis[i][-1]['cross_zx']  # [0, 0, :, 7]

                            showxz = show_xz[0, 4, 512]
                            showzx = show_zx[0, 4, 128]

                            hw1 = showxz.size(0)
                            h1 = int(hw1 ** 0.5)
                            attnxz = showxz.view(h1, -1).detach().cpu().numpy()

                            hw2 = showzx.size(0)
                            h2 = int(hw2 ** 0.5)
                            attnzx = showzx.view(h2, -1).detach().cpu().numpy()

                            image_x = tracker.image_x  # image_x
                            b1 = outputs['bbox_crop']

                            fused_img_zx = vis_heatmap(image_x[:, :, ::-1].copy(), attnzx * 40)

                            cv2.rectangle(fused_img_zx, (int(b1[0] - 0.5 * b1[2]), int(b1[1] - 0.5 * b1[3])),
                                          (int(b1[0] + 0.5 * b1[2]), int(b1[1] + 0.5 * b1[3])), (0, 0, 255), 1)

                            video_path = '/home/v-xfei/Public/vis/a_dualtfrpp/' + video.name
                            if not os.path.isdir(video_path):
                                os.makedirs(video_path)
                            cv2.imwrite(video_path + '/' + str(idx) + '_crosszx_' + str(i + 1) + '_fused.jpg', fused_img_zx)

                    if isinstance(outputs, dict):
                        pred_bbox = outputs['target_bbox']
                        bscore = outputs['best_score']

                    else:
                        pred_bbox = outputs[0]
                        bscore = outputs[1]
                    pred_bboxes.append(pred_bbox)
                    scores.append(bscore)
                toc += cv2.getTickCount() - tic
                track_times.append((cv2.getTickCount() - tic)/cv2.getTickFrequency())

                if idx == 0:
                    cv2.destroyAllWindows()
                if vis and idx > 0:
                    img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
                    gt_bbox = list(map(int, gt_bbox))
                    pred_bbox = list(map(int, pred_bbox))
                    cv2.rectangle(img, (gt_bbox[0], gt_bbox[1]),
                                  (gt_bbox[0]+gt_bbox[2], gt_bbox[1]+gt_bbox[3]), (0, 255, 0), 3)
                    cv2.rectangle(img, (pred_bbox[0], pred_bbox[1]),
                                  (pred_bbox[0]+pred_bbox[2], pred_bbox[1]+pred_bbox[3]), (0, 255, 255), 3)
                    cv2.putText(img, str(idx), (40, 40), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 255), 2)
                    cv2.imshow(video.name, img)
                    cv2.waitKey(1)
            toc /= cv2.getTickFrequency()
            # save results


if __name__ == '__main__':
    main()
